<xsl:stylesheet version = '1.0' xmlns:xsl='http://www.w3.org/1999/XSL/Transform'>

<xsl:variable name="total_SessionMessage">
    <xsl:value-of select="count(//data/SessionMessage)"/>
</xsl:variable>

<xsl:variable name="total_ProcessMessage">
    <xsl:value-of select="count(//data/ProcessMessage)"/>
</xsl:variable>

<xsl:variable name="total_UptimeMessage">
    <xsl:value-of select="count(//data/UptimeMessage)"/>
</xsl:variable>

<xsl:variable name="total_NetstatMessage">
    <xsl:value-of select="count(//data/NetstatMessage)"/>
</xsl:variable>

<xsl:variable name="total_SimpleLogMessage">
    <xsl:value-of select="count(//data/SimpleLogMessage)"/>
</xsl:variable>

<xsl:variable name="total">
    <xsl:value-of select="count(//data/*)"/>
</xsl:variable>

<xsl:template match="/bingrep">
     <HTML><HEAD><TITLE>Grep Results</TITLE></HEAD>
     <BODY>
         <H1>Grep Results</H1>
         <xsl:choose> 
             <xsl:when test="//error"> 
                 <H4>Error: <xsl:value-of select="error"/></H4>
                 <HR/>
             </xsl:when> 
             <xsl:otherwise> 
                 <B>Source binstore: </B><xsl:value-of select="sourcebin"/><BR/>
                 <B>Searching for pattern: </B><xsl:value-of select="regexp"/><BR/>
                 <B>Searching in messages: </B><xsl:value-of select="message"/><BR/>
                 <B>Searching for hosts: </B><xsl:value-of select="host"/><BR/>
                 <B>Searching for messages from agents: </B><xsl:value-of select="agent"/><BR/>
                 <B>Searching for messages of instances: </B><xsl:value-of select="instance"/><BR/>
                 <B>Searched attributes: </B><xsl:value-of select="attrs"/><BR/>
                 <HR/>
                 <xsl:apply-templates select="data"/>
                 <xsl:if test="not(data/*)">
                    <H5>No matches found.</H5>
                 </xsl:if>
             </xsl:otherwise> 
         </xsl:choose> 
     </BODY>
     </HTML>
</xsl:template>

<xsl:template match="data">
     <xsl:if test="SessionMessage"> 
             <H4>Session Messages</H4>
             <TABLE BORDER="1" CELSPACING="0" CELPADDING="0">
             <TR><TD bgcolor="#CCDEFF">STATUS</TD>
                 <TD bgcolor="#CCDEFF">HOST</TD><TD bgcolor="#CCDEFF">AGENT</TD>
                 <TD bgcolor="#CCDEFF">INSTANCE</TD><TD bgcolor="#CCDEFF">TIME</TD>
                 <TD bgcolor="#CCDEFF">USER.uid</TD><TD bgcolor="#CCDEFF">USER.username</TD>
                 <TD bgcolor="#CCDEFF">HOST.ip</TD><TD bgcolor="#CCDEFF">HOST.fqdn</TD>
                 <TD bgcolor="#CCDEFF">TTY</TD><TD bgcolor="#CCDEFF">SESSTYPE</TD>
                 <TD bgcolor="#CCDEFF">STARTTIME</TD><TD bgcolor="#CCDEFF">IDLESEC</TD></TR>
                 <xsl:apply-templates select="SessionMessage"/>     
             </TABLE>
             <p>Total: <xsl:value-of select="$total_SessionMessage"/> entries</p>
     </xsl:if> 
     <xsl:if test="ProcessMessage"> 
             <H4>Process Messages</H4>
             <TABLE BORDER="1" CELSPACING="0" CELPADDING="0">
             <TR><TD bgcolor="#CCDEFF">STATUS</TD>
                 <TD bgcolor="#CCDEFF">HOST</TD><TD bgcolor="#CCDEFF">AGENT</TD>
                 <TD bgcolor="#CCDEFF">INSTANCE</TD><TD bgcolor="#CCDEFF">TIME</TD>
                 <TD bgcolor="#CCDEFF">EOWNER.uid</TD><TD bgcolor="#CCDEFF">EOWNER.username</TD>
                 <TD bgcolor="#CCDEFF">OWNER.uid</TD><TD bgcolor="#CCDEFF">OWNER.username</TD>
                 <TD bgcolor="#CCDEFF">PID</TD><TD bgcolor="#CCDEFF">PPID</TD>
                 <TD bgcolor="#CCDEFF">CPU</TD><TD bgcolor="#CCDEFF">MEMSIZE</TD>
                 <TD bgcolor="#CCDEFF">RSS</TD><TD bgcolor="#CCDEFF">TTY</TD>
                 <TD bgcolor="#CCDEFF">STATE</TD><TD bgcolor="#CCDEFF">STARTTIME</TD>
                 <TD bgcolor="#CCDEFF">CMDLINE</TD></TR>
                 <xsl:apply-templates select="ProcessMessage"/>
             </TABLE>
             <p>Total: <xsl:value-of select="$total_ProcessMessage"/> entries</p>
     </xsl:if> 
     <xsl:if test="UptimeMessage">
             <H4>Uptime Messages</H4>
             <TABLE BORDER="1" CELSPACING="0" CELPADDING="0">
             <TR><TD bgcolor="#CCDEFF">STATUS</TD>
                 <TD bgcolor="#CCDEFF">HOST</TD><TD bgcolor="#CCDEFF">AGENT</TD>
                 <TD bgcolor="#CCDEFF">INSTANCE</TD><TD bgcolor="#CCDEFF">TIME</TD>
                 <TD bgcolor="#CCDEFF">UPTIMESEC</TD><TD bgcolor="#CCDEFF">USERS</TD>
                 <TD bgcolor="#CCDEFF">LOAD1</TD><TD bgcolor="#CCDEFF">LOAD5</TD>
                 <TD bgcolor="#CCDEFF">LOAD15</TD></TR>
                 <xsl:apply-templates select="UptimeMessage"/>
             </TABLE>
             <p>Total: <xsl:value-of select="$total_UptimeMessage"/> entries</p>
     </xsl:if>
     <xsl:if test="NetstatMessage">
             <H4>Netstat Messages</H4>
             <TABLE BORDER="1" CELSPACING="0" CELPADDING="0">
             <TR><TD bgcolor="#CCDEFF">STATUS</TD>
                 <TD bgcolor="#CCDEFF">HOST</TD><TD bgcolor="#CCDEFF">AGENT</TD>
                 <TD bgcolor="#CCDEFF">INSTANCE</TD><TD bgcolor="#CCDEFF">TIME</TD>
                 <TD bgcolor="#CCDEFF">PROTOCOL</TD><TD bgcolor="#CCDEFF">LOCAL_ADDRESS.ip</TD>
                 <TD bgcolor="#CCDEFF">LOCAL_ADDRESS.fqdn</TD><TD bgcolor="#CCDEFF">LOCAL_PORT</TD>
                 <TD bgcolor="#CCDEFF">REMOTE_ADDRESS.ip</TD>
                 <TD bgcolor="#CCDEFF">REMOTE_ADDRESS.fqdn</TD>
                 <TD bgcolor="#CCDEFF">REMOTE_PORT</TD>
                 <TD bgcolor="#CCDEFF">STATE</TD></TR>
                 <xsl:apply-templates select="NetstatMessage"/>
             </TABLE>
             <p>Total: <xsl:value-of select="$total_NetstatMessage"/> entries</p>
     </xsl:if>    
     <xsl:if test="SimpleLogMessage">
             <H4>SimpleLog Messages</H4>
             <TABLE BORDER="1" CELSPACING="0" CELPADDING="0">
             <TR><TD bgcolor="#CCDEFF">STATUS</TD>
                 <TD bgcolor="#CCDEFF">HOST</TD><TD bgcolor="#CCDEFF">AGENT</TD>
                 <TD bgcolor="#CCDEFF">INSTANCE</TD><TD bgcolor="#CCDEFF">TIME</TD>
                 <TD bgcolor="#CCDEFF">LINE</TD></TR>
                 <xsl:apply-templates select="SimpleLogMessage"/>
             </TABLE>
             <p>Total: <xsl:value-of select="$total_SimpleLogMessage"/> entries</p>
     </xsl:if>
</xsl:template>

<xsl:template match="data/*">
     <TR>
         <xsl:apply-templates/>
     </TR>
</xsl:template>

<xsl:template match="data/*//*">
     <xsl:choose> 
          <xsl:when test="*"> 
               <xsl:apply-templates/>
          </xsl:when> 
          <xsl:otherwise> 
	        <TD bgcolor="#D4EBFF"><xsl:value-of select="."/></TD>	 
          </xsl:otherwise> 
     </xsl:choose> 
</xsl:template>

</xsl:stylesheet>

