<xsl:stylesheet version = '1.0' xmlns:xsl='http://www.w3.org/1999/XSL/Transform'>

<xsl:variable name="total">
     <xsl:value-of select="count(//data/*)"/>
</xsl:variable>

<xsl:template match="/binsnapshot">
     <HTML><HEAD><TITLE>Snapshot for <xsl:value-of select="messagename"/></TITLE></HEAD>
     <BODY>
         <H1>Snapshot for <xsl:value-of select="messagename"/></H1>
         <xsl:choose> 
             <xsl:when test="error"> 
                 <H4>Error: <xsl:value-of select="error"/></H4>
                 <HR/>
             </xsl:when> 
             <xsl:otherwise> 
                 <B>Source binstore: </B><xsl:value-of select="sourcebin"/><BR/>
                 <B>Host: </B><xsl:value-of select="host"/><BR/>
                 <B>Agent: </B><xsl:value-of select="agent"/><BR/>
                 <B>Instance: </B><xsl:value-of select="instance"/><BR/>
                 <B>Snapshot is requested for time: </B><xsl:value-of select="reqtime"/><BR/>
                 <B>Nearest time is: </B><xsl:value-of select="usedtime"/><BR/>
                 <HR/>
                 <xsl:apply-templates select="data"/>
             </xsl:otherwise> 
         </xsl:choose> 
     </BODY>
     </HTML>
</xsl:template>

<xsl:template match="data">
     <TABLE BORDER="1" CELSPACING="0" CELPADDING="0">
     <xsl:if test="SessionMessage"> 
             <TR><TD bgcolor="#CCDEFF">USER.uid</TD><TD bgcolor="#CCDEFF">USER.username</TD>
                 <TD bgcolor="#CCDEFF">HOST.ip</TD><TD bgcolor="#CCDEFF">HOST.fqdn</TD>
                 <TD bgcolor="#CCDEFF">TTY</TD><TD bgcolor="#CCDEFF">SESSTYPE</TD>
                 <TD bgcolor="#CCDEFF">STARTTIME</TD><TD bgcolor="#CCDEFF">IDLESEC</TD></TR>
             <xsl:apply-templates select="SessionMessage"/>     
     </xsl:if> 
     <xsl:if test="ProcessMessage"> 
             <TR><TD bgcolor="#CCDEFF">EOWNER.uid</TD><TD bgcolor="#CCDEFF">EOWNER.username</TD>
                 <TD bgcolor="#CCDEFF">OWNER.uid</TD><TD bgcolor="#CCDEFF">OWNER.username</TD>
                 <TD bgcolor="#CCDEFF">PID</TD><TD bgcolor="#CCDEFF">PPID</TD>
                 <TD bgcolor="#CCDEFF">CPU</TD><TD bgcolor="#CCDEFF">MEMSIZE</TD>
                 <TD bgcolor="#CCDEFF">RSS</TD><TD bgcolor="#CCDEFF">TTY</TD>
                 <TD bgcolor="#CCDEFF">STATE</TD><TD bgcolor="#CCDEFF">STARTTIME</TD>
                 <TD bgcolor="#CCDEFF">CMDLINE</TD></TR>
             <xsl:apply-templates select="ProcessMessage"/>     
     </xsl:if> 
     <xsl:if test="UptimeMessage">
             <TR><TD bgcolor="#CCDEFF">UPTIMESEC</TD><TD bgcolor="#CCDEFF">USERS</TD>
                 <TD bgcolor="#CCDEFF">LOAD1</TD><TD bgcolor="#CCDEFF">LOAD5</TD>
                 <TD bgcolor="#CCDEFF">LOAD15</TD></TR>
             <xsl:apply-templates select="UptimeMessage"/>
     </xsl:if>
     <xsl:if test="NetstatMessage">
             <TR><TD bgcolor="#CCDEFF">PROTOCOL</TD><TD bgcolor="#CCDEFF">LOCAL_ADDRESS.ip</TD>
                 <TD bgcolor="#CCDEFF">LOCAL_ADDRESS.fqdn</TD><TD bgcolor="#CCDEFF">LOCAL_PORT</TD>
                 <TD bgcolor="#CCDEFF">REMOTE_ADDRESS.ip</TD>
                 <TD bgcolor="#CCDEFF">REMOTE_ADDRESS.fqdn</TD>
                 <TD bgcolor="#CCDEFF">REMOTE_PORT</TD>
                 <TD bgcolor="#CCDEFF">STATE</TD></TR>
             <xsl:apply-templates select="NetstatMessage"/>
     </xsl:if>    
     <xsl:if test="SimpleLogMessage">
             <TR><TD bgcolor="#CCDEFF">LINE</TD></TR>
             <xsl:apply-templates select="SimpleLogMessage"/>
     </xsl:if>
     </TABLE>
     <p>Total: <xsl:value-of select="$total"/> entries</p>
</xsl:template>

<xsl:template match="data/*">
     <TR>
         <xsl:apply-templates/>
     </TR>
</xsl:template>

<xsl:template match="data/*//*">
     <xsl:choose> 
          <xsl:when test="*"> 
               <xsl:apply-templates/>
          </xsl:when> 
          <xsl:otherwise> 
	        <TD bgcolor="#D4EBFF"><xsl:value-of select="."/></TD>	 
          </xsl:otherwise> 
     </xsl:choose> 
</xsl:template>

</xsl:stylesheet>

