#!/bin/ksh
# Echelog installation configuration wizard
# Author: Martin Macok <martin.macok@underground.cz>
# Version: 0.6beta
# Licence: MIT

# If you want to study this code, 
# I recommend to start from the very end of the file

#################
# CONFIGURATION #
#################

# TMPDIR=/path/to/secure_writable_dir

# path to openssl binary
OPENSSL="openssl"

FILES_USERGROUP="root.echelog"
FILES_MODE=0640

ECHELOG_CONFDIR="/etc/echelog"

POSSIBLE_CONFDIRS="/etc/echelog \
                    /usr/local/etc/echelog \
            /opt/echelog/etc/echelog \
            `pwd`/run/etc/echelog \
            `pwd`/etc/echelog \
        "

DEFAULT_UMASK="077" # OFF: "u+r"
    
DEFAULT_LOGFILE="install.log"
DEFAULT_STATEFILE="install.state"
    
DEFAULT_SERVER_PORT=8900
DEFAULT_SENDER_PERIOD=0
DEFAULT_SENDER_LIMIT=0

DEFAULT_MAGENT_NETSTAT_INT=3m
DEFAULT_MAGENT_PS_INT=3m
DEFAULT_MAGENT_W_INT=3m
DEFAULT_MAGENT_UTMP_INT=1m
DEFAULT_LOGFILE_INT=30s
    
DEFAULT_PS_AGG_SNAPSHOT=3600
DEFAULT_SESSION_AGG_SNAPSHOT=3600
DEFAULT_NETSTAT_AGG_SNAPSHOT=1800

DEFAULT_ALERT_BRIEFMSG="ALERT"
DEFAULT_ALERT_EMAIL="root@localhost"
DEFAULT_ALERT_SOUND="/usr/share/sounds/error.wav"
DEFAULT_ALERT_XDISPLAY=":0"
DEFAULT_ALERT_LOGFILE="/dev/null"

DEFAULT_ROOT_CERT_EXPIRE=365
DEFAULT_CERT_EXPIRE=365

DEFAULT_KEYLENGTHS="512 1024 2048 4096"
        
# PREDEFINED LOGFILES
        
POSSIBLE_LOG_DIRS="/var/log /var/adm"

for DIR in $POSSIBLE_LOG_DIRS ; do
    if [ -d "$DIR" ] ; then
        DEFAULT_LOG_DIR="$DIR"
        break
    fi
done

GETLOGS="syst mail http sqid inet ftpd prnt"

# TODO Solaris default maillog -> /var/log/syslog :-/
POSSIBLE_syst_LOGS=" \
                $DEFAULT_LOG_DIR/messages \
                $DEFAULT_LOG_DIR/syslog \
                $DEFAULT_LOG_DIR/log \
                $DEFAULT_LOG_DIR/secure \
                $DEFAULT_LOG_DIR/warn \
                $DEFAULT_LOG_DIR/authlog \
                $DEFAULT_LOG_DIR/sulog \
                $DEFAULT_LOG_DIR/kern \
                $DEFAULT_LOG_DIR/kernel \
                $DEFAULT_LOG_DIR/kernel/errors \
                $DEFAULT_LOG_DIR/kernel/info \
                $DEFAULT_LOG_DIR/kernel/warnings \
                "

POSSIBLE_mail_LOGS=" \
                $DEFAULT_LOG_DIR/maillog \
                $DEFAULT_LOG_DIR/mail \
                $DEFAULT_LOG_DIR/mail/info \
                $DEFAULT_LOG_DIR/mail/maillog \
                $DEFAULT_LOG_DIR/mail/warnings \
                $DEFAULT_LOG_DIR/mail/errors \
                "

POSSIBLE_http_LOGS=" \
                $DEFAULT_LOG_DIR/httpd/access_log \
                $DEFAULT_LOG_DIR/httpd/error_log \
                $DEFAULT_LOG_DIR/httpd/ssl-access_log \
                $DEFAULT_LOG_DIR/httpd/ssl-error_log \
                $DEFAULT_LOG_DIR/boa/access_log \
                $DEFAULT_LOG_DIR/boa/error_log \
                "

POSSIBLE_sqid_LOGS=" \
                $DEFAULT_LOG_DIR/squid/access.log \
                $DEFAULT_LOG_DIR/squid/cache.log \
                $DEFAULT_LOG_DIR/squid/errors.log \
                $DEFAULT_LOG_DIR/squid/error.log \
                "

POSSIBLE_inet_LOGS=" \
                $DEFAULT_LOG_DIR/xinetd.log \
                $DEFAULT_LOG_DIR/inetd.log \
                $DEFAULT_LOG_DIR/inetd.sec \
                "

POSSIBLE_ftpd_LOGS=" \
                $DEFAULT_LOG_DIR/xferlog \
                $DEFAULT_LOG_DIR/vsftpd.log \
                "

POSSIBLE_prnt_LOGS=" \
                $DEFAULT_LOG_DIR/cups/access_log \
                $DEFAULT_LOG_DIR/cups/error_log \
                $DEFAULT_LOG_DIR/cups/page_log \
                $DEFAULT_LOG_DIR/lpr/errors \
                $DEFAULT_LOG_DIR/lpr/info \
                $DEFAULT_LOG_DIR/lpr/warnings \
                $DEFAULT_LOG_DIR/lp/log \
                $DEFAULT_LOG_DIR/lp/lpd.log \
                "

#############
# FUNCTIONS #
#############

# stderr and log
function error  { echo "ERROR: $@" | tee -a "$LOGFILE" | cat >&2 ; }

# err to screen and to log, exit
function abort
{ echo "ABORT: $@" | tee -a "$LOGFILE" | cat >&2 ; 
  cleanup
  exit 1 ;
}

# only to log file
function log { [ -z "$@" ] || echo "$@" >>"$LOGFILE" ; }

# to both screen and log file
function echolog { echo "$@" | tee -a "$LOGFILE" ; }

# usage: make_temp [-d] MKTEMP_ARG_FILE
# returns 0 on success, 1 on failure
function make_temp
{
    if [ -x "`which mktemp 2>/dev/null`" ] ; then
        mktemp "$@" && return 0
        return 1
    fi

    if [ ! -z "$TMPDIR" ] && \
       [ "$TMPDIR" != "/tmp" -a "$TMPDIR" != "/var/tmp" ] && \
       [ -d "$TMPDIR" -a -w "$TMPDIR" ]; then
        if [ "x$1" = "x-d" ] ; then
            MAKEIT=mkdir
            shift
        else
            MAKEIT=touch
        fi
        TF="${@}.$$"
        $MAKEIT "$TF" && echo "$TF" && return 0
        echo "ERROR: Can't create temporary file $TF" >&2
        return 1
    else
        echo >&2
        echo "You should install mktemp from ftp://ftp.mktemp.org/pub/mktemp/" >&2
        echo "See Echelog documentation (Requirements)" >&2
        echo >&2
        echo "OR">&2
        echo >&2
        echo "Change \$TMPDIR environment variable to secure writable directory" >&2
        echo "and do the same for alert if you use it." >&2
        return 1
    fi
}

# creates temporary file and installation logfile
function init_conf
{
    umask "$DEFAULT_UMASK"
    LC_ALL=C
    LANG=C

    TMPFILE=$(make_temp "${TMPDIR:-/tmp}/init_config.sh.tmp$$.XXXXXX")
    [ -z "$TMPFILE" ] && \
        echo "Cannot create temporary file in ${TMPDIR:-/tmp}, \
set correct \$TMPDIR variable" && exit
    LOGFILE=$(make_temp "${TMPDIR:-/tmp}/init_config.log.XXXXXX")
    [ -z "$LOGFILE" ] && \
        echo "Cannot create temporary file in ${TMPDIR:-/tmp}, \
set correct \$TMPDIR variable" && exit
    log "STARTING at `date` =============================================="
    trap cleanup 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15

    if [ ! -x "`which openssl 2>/dev/null`" ] ; then
        echo
        echolog "Warning: openssl binary not found in \$PATH, you can't generate keys"
        echo
        yes_or_no "Continue" ANSWER
        [ "$ANSWER" = no ] && exit

        OPENSSL_NOT_FOUND=1
    fi
}

function cleanup
{
    rm -f $TMPFILE
    echo "Installation log: $LOGFILE"
    exit
}

function intro
{
    cat <<EOF

             >>  Welcome to Echelog configuration  <<

EOF
#    yes_or_no "Is Echelog already installed" ANSWER
#    [ "$ANSWER" = no ] && \
#        echo "Install Echelog and then run again." && exit
}


# asks user for yes/no and set the answer to VAR
# usage: yes_or_no "Question" VAR
function yes_or_no
{
    MSG="$1"
    VAR="$2"
    while true ; do
        echo -n "$MSG (Y/n) ? "
        read ANSWER
        case "x$ANSWER" in
            xy|xY|xyes|xYES|x)
                eval "$VAR"=yes
                log "Q: $MSG A: yes"
                return
                ;;
            xn|xN|xno|xNO)
                eval "$VAR"=no
                log "Q: $MSG A: no"
                return
                ;;
            xabort)
                abort "User choosed to abort ($MSG)"
                ;;
            *)
                error "Invalid response ($ANSWER)"
                continue
                ;;
        esac
    done
}

function my_seq
{
    if [ -x "`which seq 2>/dev/null`" ] ; then
        seq "$@" && return
    fi

    if [ $# -gt 2 ] ; then
        shift $(($# - 2))
    fi

    FROM="$1"
    TO="$2"
    while [ "$FROM" -le "$TO" ] ; do
        echo "$FROM"
        FROM=$(($FROM + 1))
    done
}

# asks user for a choise of several alternatives
# usage: choose "Question" VAR opt1 opt2 [...]
function choose 
{
    MSG="$1"
    VAR="$2"
    shift 2
    echo "$MSG"
    for I in `my_seq -w 1 $#` ; do eval echo "\ $I\) \$$I" ; done
    while true ; do
        echo -n "Your choice (1-$#) ? "
        read ANSWER
        case "x$ANSWER" in
            x[1-$#])
                eval "$VAR"=\"\$$ANSWER\"
                eval ANS=\"\$$ANSWER\"
                log "Q: $MSG A: $ANS"
                return
                ;;
            xabort)
                abort "User choosed to abort"
                ;;
            x)
                continue
                ;;
            *)
                error "Invalid response ($ANSWER)"
                continue
                ;;
        esac
    done
}

# usage: ask_string MSG VAR
function ask_string
{
    MSG="$1"
    VAR="$2"
    shift 2
    while true ; do
        echo -n "$MSG: "
        read ANSWER
        case "x$ANSWER" in
            x)
                continue
                ;;
            xabort)
                abort "User choosed to abort"
                ;;
            *)
                eval "$VAR"=\"$ANSWER\"
                log "Q: $MSG A: $ANSWER"
                return
                ;;
        esac
    done
}

# usage: ask_string_default DEFAULT MSG VAR
function ask_string_default
{
    DEFAULT="$1"
    MSG="$2"
    VAR="$3"
    shift 2
    while true ; do
        echo -n "$MSG [$DEFAULT]: "
        read ANSWER
        case "x$ANSWER" in
            x)
                eval "$VAR"=\"$DEFAULT\"
                log "Q: $MSG A: $DEFAULT (default)"
                return
                ;;
            xabort)
                abort "User choosed to abort"
                ;;
            *)
                eval "$VAR"=\"$ANSWER\"
                log "Q: $MSG A: $ANSWER"
                return
                ;;
        esac
    done
}

# usage: ask_num MSG VAR
function ask_num
{
    MSG="$1"
    VAR="$2"
    while true ; do
        ask_string "$@"
        eval NUM=\"\$$VAR\"
        if expr "$NUM" : "[0-9][0-9]*$" >/dev/null ; then
            return
        else
            error "$NUM is not a number"
        fi
    done
}

# usage: ask_num_default DEFAULT MSG VAR
function ask_num_default
{
    DEFAULT="$1"
    MSG="$2"
    VAR="$3"
    while true ; do
        ask_string_default "$@"
        eval NUM=\"\$$VAR\"
        if expr "$NUM" : "[0-9][0-9]*$" >/dev/null ; then
            return
        else
            error "$NUM is not a number"
        fi
    done
}

# usage: test_host HOSTNAME
# returns 0 on succes, 1 on failure
function test_host
{
   if [ -x "`which host 2>/dev/null`" ] ; then
        if  host "$HOSTNAME" 2>&1 | grep -i "not found" >/dev/null ; then
            return 1
        else
            return 0
        fi
    elif [ -x "`which nslookup 2>/dev/null`" ] ; then
        if  nslookup "$HOSTNAME" 2>&1 | \
                grep -i " can't find.*:.*domain" >/dev/null ; then
            return 1
        else
            return 0
        fi
    fi
    # I don't care, your system sucks
    return 0
}

# usage: ask_existing_hostname MSG VAR
function ask_existing_hostname
{
    MSG="$1"
    VAR="$2"
    while true ; do
        ask_string "$@"
        eval HOSTNAME=\"\$$VAR\"
        if ! test_host "$HOSTNAME" ; then
            error "Hostname $HOSTNAME doesn't exist"
            echolog "You can add \"IP $HOSTNAME\" to /etc/hosts (if not there already)."
            yes_or_no "Do you really want to use $HOSTNAME" ANSWER
            if [ "$ANSWER" = yes ] ; then
                return
            fi
        else
            return
        fi
    done
}

# usage: ask_existing_hostname_default DEFAULT MSG VAR
function ask_existing_hostname_default
{
    DEFAULT="$1"
    MSG="$2"
    VAR="$3"
    while true ; do
        ask_string_default "$@"
        eval HOSTNAME=\"\$$VAR\"
        if ! test_host "$HOSTNAME" ; then
            error "Hostname $HOSTNAME doesn't exist"
            echolog "You can add \"IP $HOSTNAME\" to /etc/hosts."
            yes_or_no "Do you really want to use $HOSTNAME" ANSWER
            if [ "$ANSWER" = yes ] ; then
                return
            fi
        else
            return
        fi
    done
}

# asks a value, test if it is a directory and set a VAR
# usage: ask_existing_dir "Question" VAR
function ask_existing_dir
{
    MSG="$1"
    VAR="$2"
    while true ; do
        echo -n "$MSG (DIR): "
        read ANSWER
        if [ "$ANSWER" = "abort" ] ; then
                abort "User choosed to abort when choosing $MSG"
        elif [ -d "$ANSWER" ] ; then
            eval "$VAR"=\"$ANSWER\"
            eval ANS=\"$ANSWER\"
            log "Q: $MSG A: $ANS"
            return
        else
            error "$ANSWER is not a directory"
        fi
    done
}

# asks a value, test if it is a directory and set a VAR
# usage: ask_existing_dir_default DEFAULT "Question" VAR
function ask_existing_dir_default
{
    DEFAULT="$1"
    MSG="$2"
    VAR="$3"
    while true ; do
        echo -n "$MSG (DIR) [$DEFAULT]: "
        read ANSWER
        if [ "$ANSWER" = "abort" ] ; then
                abort "User choosed to abort when choosing $MSG"
        fi
        if [ -z "$ANSWER" ] ; then
                ANSWER="$DEFAULT"
        fi
        if [ -d "$ANSWER" ] ; then
            eval "$VAR"=\"$ANSWER\"
            eval ANS=\"$ANSWER\"
            log "Q: $MSG A: $ANS"
            return
        else
            error "$ANSWER is not a directory"
        fi
    done
}

# asks a value, test if it is a directory and set a VAR
# usage: ask_existing_dir "Question" VAR
function ask_existing_writable_dir
{
    MSG="$1"
    VAR="$2"
    while true ; do
        ask_existing_dir "$@"
        eval DIR=\"\$$VAR\"
        if test -w "$DIR" ; then
            return ;
        else
            error "Cannot write to $DIR directory"
        fi
    done
}

# asks a value, test if it is a writable file and set a VAR
# usage: ask_writable_fname "Question" VAR
function ask_writable_fname
{
    MSG="$1"
    VAR="$2"
    while true ; do
        echo -n "$MSG (FILENAME): "
        read ANSWER
        if [ "$ANSWER" = "abort" ] ; then
            abort "User choosed to abort when choosing the filename."
        elif touch "$ANSWER" 2>/dev/null ; then
            eval "$VAR"=\"$ANSWER\"
            log "Q: $MSG A: $ANSWER"
            return
        else
            error "Cannot create $ANSWER file"
        fi
    done
}

# asks a value, test if it is a file and set a VAR
# usage: ask_existing_fname "Question" VAR
function ask_existing_fname
{
    MSG="$1"
    VAR="$2"
    while true ; do
        echo -n "$MSG (FILENAME): "
        read ANSWER
        if [ "$ANSWER" = "abort" ] ; then
            abort "User choosed to abort when choosing the filename."
        elif test -f "$ANSWER" 2>/dev/null ; then
            eval "$VAR"=\"$ANSWER\"
            log "Q: $MSG A: $ANSWER"
            return
        else
            error "File $ANSWER doesn't exist"
        fi
    done
}

# asks a value, test if it is a file and set a VAR
# usage: ask_existing_fname_default DEFAULT "Question" VAR
function ask_existing_fname_default
{
    DEFAULT="$1"
    MSG="$2"
    VAR="$3"
    while true ; do
        echo -n "$MSG (FILENAME) [$DEFAULT]: "
        read ANSWER
        if [ -z "$ANSWER" ] ; then
            ANSWER="$DEFAULT"
        fi
        if [ "$ANSWER" = "abort" ] ; then
            abort "User choosed to abort when choosing the filename."
        elif test -f "$ANSWER" 2>/dev/null ; then
            eval "$VAR"=\"$ANSWER\"
            log "Q: $MSG A: $ANSWER"
            return
        else
            error "File $ANSWER doesn't exist"
        fi
    done
}

# usage: ask_fname "Question" VAR
function ask_fname
{
    MSG="$1"
    VAR="$2"
    while true ; do
        echo -n "$MSG (FILENAME): "
        read ANSWER
        [ -z "$ANSWER" ] && continue
        if [ "$ANSWER" = "abort" ] ; then
            abort "User choosed to abort when choosing the filename."
        fi
        eval "$VAR"=\"$ANSWER\"
        log "Q: $MSG A: $ANSWER"
        return
    done
}

# searches the directory with echelog configuration
function search_echelog_confdir
{
    echolog "Searching for configuration directory ..."
    log "Testing $POSSIBLE_CONFDIRS"
    for I in $POSSIBLE_CONFDIRS ; do
        if test_echelog_confdir "$I" >/dev/null 2>&1 ; then
            ECHELOG_CONFDIR="$I"
            return 0
        fi
    done
    return 1
}

# test if ECHELOG_CONFDIR contains all needed files and is writable
# usage: if test_echelog_confdir DIR ; then OK ...
function test_echelog_confdir
{
    DIR="$1"
    if cd "$DIR" 2>/dev/null ; then 
        if  test -f common.conf && \
            test -f echelog.conf && \
            test -f messages.conf && \
            test -f types.conf && \
            test -d modules && \
            test -d templates ; then
            if test -w echelog.conf && test -w common.conf && test -w . ; then
                return 0
            else
                error "$DIR not writable"
                return 1
            fi
         else
            error "$DIR is not complete" 
            return 1
         fi
    fi

    error "$DIR doesn't exist" 
    return 1
}

# try to finds ECHELOG_CONFDIR, if failes, ask a user. check it also
function set_echelog_confdir
{
    OLDDIR="$PWD"
    if test_echelog_confdir "$ECHELOG_CONFDIR" >/dev/null 2>&1 ; then
        return
    fi
    if search_echelog_confdir ; then
        yes_or_no "Configuration directory is $ECHELOG_CONFDIR" ANSWER
        if [ "$ANSWER" = yes ] ; then
            cd "$OLDDIR"
            return
        fi
    fi
    while true ; do
        ask_existing_writable_dir "Configuration directory" ECHELOG_CONFDIR
        if test_echelog_confdir "$ECHELOG_CONFDIR" ; then
            break
        fi
        error "$ECHELOG_CONFDIR is not correct or is not writable"
    done
    cd "$OLDDIR"
}

function set_logfile
{
    NEWLOG="${ECHELOG_CONFDIR}/${DEFAULT_LOGFILE}"
#    [ -f "$NEWLOG" ] && grep '^FINISHED$' <"$NEWLOG" >/dev/null && \
#        rm -f "$NEWLOG"
    append_file "$LOGFILE" "$NEWLOG"
    LOGFILE="$NEWLOG"
}
    
function set_all_paths
{
    log "Configuration directory set to $ECHELOG_CONFDIR"
    TEMPLATES_DIR="${ECHELOG_CONFDIR}/templates"
    MODULES_DIR="$ECHELOG_CONFDIR/modules"
    KEYS_DIR="$ECHELOG_CONFDIR/modules/keys"
    mkdir -p "$TEMPLATES_DIR" "$MODULES_DIR" "$KEYS_DIR"
}

function set_installation_state
{
    STATE_FILE="${ECHELOG_CONFDIR}/${DEFAULT_STATEFILE}"
    if [ -f "$STATE_FILE" ] ; then
        echolog "Reading installation state file ($STATE_FILE)"
        . "$STATE_FILE"
    fi
}

# usage: section cmd [args ..]
function section
{
    NAME="${1}_FINISHED"
    eval SECTION="\$$NAME"

    if [ -z "$SECTION" ] ; then
        log "   -------------(section $1 started)-------------"
        "$@"
        log "   -------------(section $1 finished)------------"
        # do it again (section could be called recurently)
        NAME="${1}_FINISHED"
        append_to_file "${NAME}=1" "$STATE_FILE"
    else
        echolog "Skipping already finished part ($@)"
    fi
}

# usage: save_state VARNAME
function save_state
{
    eval VALUE="\"\$$1\""
    append_to_file "${1}=\"$VALUE\"" "$STATE_FILE"
}

function reset_state
{
    rm -f "$STATE_FILE"
}

# if BAKFILE defined, ask user if he wants to restore from it
function restore_configuration
{
    [ -z "$BAKFILE" ] && return
    [ -f "$BAKFILE" ] || return

    yes_or_no "Do you want to restore from $BAKFILE" ANSWER
    [ "$ANSWER" = no ] && return

    log "Restoring from $BAKFILE"
    ( cd / && gunzip -c "$BAKFILE" | tar xf - ; ) || \
                   abort "Cannot restore from $BAKFILE"
    echolog "Restored - please run the script again"
    reset_state
    cleanup
}

# ask user if wants to backup specified files to ARCHIVE
# usage: backup_files ARCHIVE FILE [FILE2 DIR3 ...]
function backup_files
{
    BAKFILE="$1"
    shift
    while true ; do
        yes_or_no "Backup $@" DOBAK
        if [ "$DOBAK" = no ] ; then
            BAKFILE=""
            return
        fi
        ask_string_default "$BAKFILE" "Where to backup" BAKFILE
        log "Backing up $@ to $BAKFILE"
        tar cf - "$@" | gzip -c - >"$BAKFILE" && return
        error "Cannot backup to $BAKFILE"
    done
}

function backup_configuration
{
        backup_files ~/echelog_confdir_orig.tgz "$ECHELOG_CONFDIR"
        [ -z "$BAKFILE" ] || save_state BAKFILE
}

# appends file1 to the end of file2
# usage: append_file file1 file2
function append_file
{
    log "Appending file $1 to the end of $2"
    cat "$1" >>"$2" || abort "Cannot append $1 to $2 file"
}

# usage: append_file file1 file2
function append_to_file
{
    log "Appending \"$1\" string to the end of $2"
    echo "$1" >>"$2" || abort "Cannot append $1 to $2 file"
}

# inserts file1 in file2 at POS
# POS is a string which is #-comment in file2
# usage: insert_file file1 file2 POS
function insert_file
{
    SOURCE="$1"
    DEST="$2"
    POS="$3"
    log "Inserting file $SOURCE into file $DEST at position with # $POS"
    ( [ -r "$SOURCE" ] && [ -w "$DEST" ] ) || \
       abort "Not enough permissions to work with $SOURCE and $DEST files"
        # test if POS exactly once only there
    [ $(grep -c "^#\ *$POS" <"$DEST" ) -eq 1 ] || \
       abort "There should be exactly 1 line with \"# $POS\" in $DEST"
    sed -n "1,/^#\ *$POS/p" <"$DEST" >"$TMPFILE"
    cat "$SOURCE" >>"$TMPFILE"
        # source could end with a non-newline, so to make sure ->
    echo >>"$TMPFILE"
    sed -n "/^#\ *$POS/,\$p" <"$DEST" >>"$TMPFILE"
    cat "$TMPFILE" >"$DEST"
}

# remove section from file which was inserted with insert_file
# exactly: removes what is between "^# POS" lines from file
# usage: remove_from_file file POS
function remove_from_file
{
    FILE="$1"
    POS="$2"
    log "Removing section $POS from file $FILE"
    [ -r "$FILE" -a -w "$FILE" ] || \
        abort "Not enough permissions to change $FILE file"
    [ $(grep -c "$POS" <"$FILE") -eq 2 ] || \
        abort "There should be exactly 2 lines with \"# $POS\" in $FILE"
    #sed -n "1,/^#\ *$POS/p" <"$FILE" | grep -v "^#\ *$POS" >"$TMPFILE"
    echo -n >"$TMPFILE"
    cat "$FILE" | { while read LINE ; do # copy beginning of file
        if expr "$LINE" : "#\ *$POS" >/dev/null ; then
            break
        else
            echo "$LINE" >>"$TMPFILE"
        fi
        done
        while read LINE ; do # don't copy removed section
        if expr "$LINE" : "#\ *$POS" >/dev/null ; then
            break
        fi
        done
        echo "# $POS" >>"$TMPFILE"
        while read LINE ; do
            echo "$LINE" >>$TMPFILE
        done }
    cat "$TMPFILE" >"$FILE"
}
                
    

# substitutes WHAT with WITH in FILE
# usage: sed_file WHAT WITH FILE
function sed_file
{
    WHAT="$1"
    WITH="$2"
    FILE="$3"
    log "Substituting $WHAT with $WITH in file $FILE"
    [ -r "$FILE" -a -w "$FILE" ] || \
        abort "Can't change $FILE file"
    grep "$WHAT" <"$FILE" >/dev/null || \
        echolog "Warning: $WHAT not found in $FILE"
    # could be done with "ed" in place
    sed "s#${WHAT}#${WITH}#" <"$FILE" >"$TMPFILE" 
    cat "$TMPFILE" >"$FILE"
}

# copies file1 to file2
# usage: copy_file file1 file2
function copy_file
{
    SOURCE="$1"
    DEST="$2"
    log "Copying file $SOURCE to $DEST"
    cat "$SOURCE" >"$DEST" || \
       abort "Cannot copy $SOURCE to $DEST file."
}

# usage: move_file file1 file2
function move_file
{
    SOURCE="$1"
    DEST="$2"
    log "Moving file $SOURCE to $DEST"
    mv "$SOURCE" "$DEST" || \
       abort "Cannot move $SOURCE to $DEST file."
}

# removes file
# usage: remove_file FILE
function remove_file
{
    FILE="$1"
    log "Removing file $FILE"
    rm "$FILE" || abort "Cannot remove $FILE"
}

# usage: zero_file FILE
function zero_file
{
    FILE="$1"
    log "Zeroing $FILE"
    echo -n >"$FILE" || abort "Cannot zero $FILE"
}

# removes directory, asks if not empty
# usage: remove_dir DIR
function remove_dir
{
    FILE="$1"
    log "Removing directory $FILE"
    [ -d "$FILE" ] || abort "$FILE not a directory"
    if rmdir "$FILE" ; then
        return
    else
        yes_or_no "Cannot remove directory, \
do you want to try rm -rf $FILE" ANSWER
            if [ "$ANSWER" = yes ] ; then
            rm -rf "$FILE"
        fi
    fi
}

# usage: set_string MSG MARK FILE
function set_string
{
    MSG="$1"
    MARK="$2"
    FILE="$3"

    ask_string "$MSG" VALUE
    sed_file "$MARK" "$VALUE" "$FILE"
}

# usage: set_existing_hostname MSG MARK FILE
function set_existing_hostname
{
    MSG="$1"
    MARK="$2"
    FILE="$3"

    ask_existing_hostname "$MSG" VALUE
    sed_file "$MARK" "$VALUE" "$FILE"
}

# usage: set_existing_hostname_default DEFAULT MSG MARK FILE
function set_existing_hostname_default
{
    DEFAULT="$1"
    MSG="$2"
    MARK="$3"
    FILE="$4"

    ask_existing_hostname_default "$DEFAULT" "$MSG" VALUE
    sed_file "$MARK" "$VALUE" "$FILE"
}

# usage: set_existing_fname_default DEFAULT MSG MARK FILE
function set_existing_fname_default
{
    DEFAULT="$1"
    MSG="$2"
    MARK="$3"
    FILE="$4"

    ask_existing_fname_default "$DEFAULT" "$MSG" VALUE
    sed_file "$MARK" "$VALUE" "$FILE"
}

# usage: set_num MSG MARK FILE
function set_num
{
    MSG="$1"
    MARK="$2"
    FILE="$3"

    ask_num "$MSG" VALUE
    sed_file "$MARK" "$VALUE" "$FILE"
}

# usage: set_num_default DEFAULT MSG MARK FILE
function set_num_default
{
    DEFAULT="$1"
    MSG="$2"
    MARK="$3"
    FILE="$4"

    ask_num_default "$DEFAULT" "$MSG" VALUE
    sed_file "$MARK" "$VALUE" "$FILE"
}

# usage: set_string_default DEFAULT MSG MARK FILE
function set_string_default
{
    DEFAULT="$1"
    MSG="$2"
    MARK="$3"
    FILE="$4"

    ask_string_default "$DEFAULT" "$MSG" VALUE
    sed_file "$MARK" "$VALUE" "$FILE"
}

# usage: set_interval_default DEFAULT MSG FILE
function set_interval_default
{
    DEFAULT="$1"
    MSG="$2"
    FILE="$3"
    while true ; do
        ask_string_default "$DEFAULT" "$MSG" VALUE
        if expr "$VALUE" : "[0-9][0-9]*[mM]$" >/dev/null ; then
            VALUE=$(echo $VALUE|tr -d [a-zA-Z])
            sed_file XINTERVALS 0 "$FILE"
            sed_file XINTERVALM "$VALUE" "$FILE"
            sed_file XINTERVALH 0 "$FILE"
            return
        elif expr "$VALUE" : "[0-9][0-9]*[hH]$" >/dev/null ; then
            VALUE=$(echo $VALUE|tr -d [a-zA-Z])
            sed_file XINTERVALS 0 "$FILE"
            sed_file XINTERVALM 0 "$FILE"
            sed_file XINTERVALH "$VALUE" "$FILE"
            return
        elif expr "$VALUE" : "[0-9][0-9]*[sS]*$" >/dev/null ; then
            VALUE=$(echo $VALUE|tr -d [a-zA-Z])
            sed_file XINTERVALS "$VALUE" "$FILE"
            sed_file XINTERVALM 0 "$FILE"
            sed_file XINTERVALH 0 "$FILE"
            return
        else
            error "Invalid response: (use <num>, <num>s, <num>m or <num>h)"
        fi
    done
}

function add_to_modules
{
    log "Adding $@ to ${TEMPLATES_DIR}/echelog_modules.conf"
    echo "$@" >>"${TEMPLATES_DIR}/echelog_modules.conf"
}

function add_to_dataflow
{
    log "Adding $@ to ${TEMPLATES_DIR}/echelog_dataflow.conf"
    echo "$@" >>"${TEMPLATES_DIR}/echelog_dataflow.conf"
}

function setup_dh_params
{
    if [ -f "$DH_PARAMS" ] ; then
        yes_or_no "Use existing $DH_PARAMS DH parameters file" ANSWER
        if [ "$ANSWER" = yes ] ; then
            return
        fi
    fi
    
    yes_or_no "Do you already have DH parameters file" ANSWER
    if [ "$ANSWER" = yes ] ; then
        ask_existing_fname "DH parameters file" FN
        copy_file "$FN" "$DH_PARAMS"
        return
    fi
    
    set_keylength

    echolog "Generating DH params $DH_PARAMS"

    "$OPENSSL" dhparam "$KEYLENGTH" -out "$DH_PARAMS" || \
       abort "Generating dhparam failed"
}

function setup_dsa_params
{
    if [ -f "$DSA_PARAMS" ] ; then
        yes_or_no "Use existing $DSA_PARAMS DSA parameters file" ANSWER
        if [ "$ANSWER" = yes ] ; then
            return
        fi
    fi
    
    yes_or_no "Do you already have DSA parameters file" ANSWER
    if [ "$ANSWER" = yes ] ; then
        ask_existing_fname "DSA parameters file" FN
        copy_file "$FN" "$DSA_PARAMS"
        return
    fi

    echolog "Generating DSA params $DSA_PARAMS"
    
    set_keylength

    "$OPENSSL" dsaparam "$KEYLENGTH" -out "$DSA_PARAMS" || \
      abort "Generating dsaparam failed"
}

function setup_root_certificate_public
{
    [ "$ROOT_CERTIFICATE_GENERATED" = 1 ] && return
    
    if [ -f "$ROOT_CERT" ] ; then
        yes_or_no "Use existing $ROOT_CERT ROOT certificate" ANSWER
        if [ "$ANSWER" = yes ] ; then
            return
        fi
    fi
    yes_or_no "Do you already have ROOT certificate file" ANSWER
    if [ "$ANSWER" = yes ] ; then
        ask_existing_fname "ROOT certificate file" FN
        copy_file "$FN" "$ROOT_CERT"
        return
    fi

    setup_root_certificate
}

function setup_root_certificate
{
     [ "$ROOT_CERTIFICATE_GENERATED" = 1 ] && \
             echolog "ROOT certificate already generated" && return
    if [ -f "$ROOT_CERT" -a -f "$ROOT_PRIV_KEY" ] ; then
        yes_or_no "Use existing $ROOT_CERT ROOT certificate and $ROOT_PRIV_KEY ROOT private key" ANSWER
        if [ "$ANSWER" = yes ] ; then
            return
        fi
    fi
    
    yes_or_no "Do you already have ROOT certificate file and root private key" ANSWER
    if [ "$ANSWER" = yes ] ; then
        ask_existing_fname "ROOT certificate file" FN
        copy_file "$FN" "$ROOT_CERT"
        ask_existing_fname "ROOT private key file" ROOT_PRIV_KEY
        return
    fi

    echolog "Generating new ROOT certificate with corresponding private key"

    ask_num_default "$DEFAULT_ROOT_CERT_EXPIRE" \
            "ROOT certificate will expire in (days)" EXPIRE_DAYS
    "$OPENSSL" req -newkey dsa:"$DSA_PARAMS" -keyout "$ROOT_PRIV_KEY" \
              -new -x509 -days "$EXPIRE_DAYS" -nodes -out "$ROOT_CERT" || \
              abort "Generating ROOT keys failed"

    echolog "ROOT CERTIFICATE: $ROOT_CERT"
    echolog "ROOT PRIVATE KEY: $ROOT_PRIV_KEY"

    ROOT_CERTIFICATE_GENERATED=1
    save_state ROOT_CERTIFICATE_GENERATED
}

function setup_my_certificate
{
    if [ -f "$CERT" -a -f "$PRIV_KEY" ] ; then
        yes_or_no "Use existing $CERT certificate and $PRIV_KEY private key" ANSWER
        if [ "$ANSWER" = yes ] ; then
            return
        fi
    fi
    
    yes_or_no \
     "Do you already have ${KEYS_TARGET}'s signed certificate file and private key" ANSWER
    if [ "$ANSWER" = yes ] ; then
        ask_existing_fname "Signed certificate file" FN
        copy_file "$FN" "$CERT"
        ask_existing_fname "Private key file" FN
        copy_file "$FN" "$PRIV_KEY"
        return
    fi

    section setup_dsa_params
    setup_root_certificate

    echolog "Generating $KEYS_TARGET certificate and private key"
    
    echo
    echolog "WARNING: You MUST set \"Common Name\" for $KEYS_TARGET key!"
    echo
    
    ask_num_default "$DEFAULT_CERT_EXPIRE" \
             "Certificate will expire in (days)" EXPIRE_DAYS
    "$OPENSSL" req -nodes -newkey dsa:"$DSA_PARAMS" -keyout "$PRIV_KEY" \
              -new -days "$EXPIRE_DAYS" -out "${CERT}.req" || \
              abort "Generating keys failed"

    "$OPENSSL" x509 -days "$EXPIRE_DAYS" -CA "$ROOT_CERT" -CAkey "$ROOT_PRIV_KEY" \
               -req -CAcreateserial -CAserial "$KEYS_DIR/ca.srl" -in "${CERT}.req" \
               -out "${CERT}" || abort "Signing new certificate with ROOT failed"

}


function clean_root_private_key
{
    echo
    echolog "WARNING: You shouldn't leave ROOT private key on unsafe places."
    echo "If you have already signed all you senders and receivers keys,"
    echo "you should delete certificate key or move it to some safe location."
    echo
    yes_or_no "Delete $ROOT_PRIV_KEY file from this machine" ANSWER
    if [ "$ANSWER" = "yes" ] ; then
        remove_file "$ROOT_PRIV_KEY"
    else
        yes_or_no "Move $ROOT_PRIV_KEY file somewhere" ANSWER
        if [ "$ANSWER" = "yes" ] ; then
            ask_writable_fname "Destination file" DESTFILE
            move_file "$ROOT_PRIV_KEY" "$DESTFILE"
        else
            echolog "You have been warned."
        fi
    fi
}

function set_keylength
{
    choose "Keylength in bits (1024 or more recommended)" KEYLENGTH $DEFAULT_KEYLENGTHS
}

function setup_keys_sender
{
    setup_keys sender
}

function setup_keys_receiver
{
    setup_keys receiver
}

# ask for existing params/keys/certs or generate new on demand
# usage: setup_keys [sender|receiver]
function setup_keys
{
    KEYS_TARGET="$1"
    DH_PARAMS="$KEYS_DIR/dh_params.pem"
    DSA_PARAMS="$KEYS_DIR/dsa_params.pem"
    PRIV_KEY="$KEYS_DIR/${KEYS_TARGET}_dsa_key.pem"
    CERT="$KEYS_DIR/${KEYS_TARGET}_certificate.pem"
    ROOT_CERT="$KEYS_DIR/root_certificate.pem"
    ROOT_PRIV_KEY="$KEYS_DIR/root_dsa_key.pem"

    [ "$OPENSSL_NOT_FOUND" = 1 ] && return
    
    yes_or_no "Do you want to setup keys for $KEYS_TARGET" ANSWER
    [ "$ANSWER" = "no" ] && return

    if [ "$KEYS_TARGET" = "receiver" ] ; then
        section setup_dh_params
    fi

    setup_my_certificate
    setup_root_certificate_public

    chown_files "key files" \
             "$DH_PARAMS" "$DSA_PARAMS" "$PRIV_KEY" "$CERT" "$ROOT_CERT"

    if [ "$KEYS_TARGET" = "sender" -a "$GLOBALTARGET" != both ] || \
       [ "$KEYS_TARGET" = "receiver" -a "$GLOBALTARGET" != proxy ] ; then
        clean_root_private_key
    fi

}

# usage: chown_files FILE_NAME_MSG FILE [ FILE ... ]
function chown_files
{
    [ `whoami` != root ] && return

    MSG="$1"
    shift

    yes_or_no "Change ownership of $MSG to $FILES_USERGROUP($FILES_MODE)" \
                                                                     ANSWER
    [ "$ANSWER" = no ] && return

    chown -f "$FILES_USERGROUP" "$@"
    chmod -f "$FILES_MODE" "$@"
}

# installs sender
function install_sender
{
    echolog "Configuring sender"

    SENDER_TEMPLATE="${TEMPLATES_DIR}/sender.conf"
    
    section setup_keys_sender

    set_existing_hostname "Where to send gathered data (receiver hostname)" \
                          XRECEIVERHOSTNAME "$SENDER_TEMPLATE"
    
    set_num_default "$DEFAULT_SERVER_PORT" \
                    "Server (receiver) port" XRECEIVERPORT "$SENDER_TEMPLATE"

    set_string "Receiver certificate Common Name" \
               XRECEIVERCOMMONNAME "$SENDER_TEMPLATE"

    add_to_modules "Module sender  Sender  \"modules/sender.conf\""

    add_to_dataflow "Add sender"

    copy_file "$SENDER_TEMPLATE" "${MODULES_DIR}/sender.conf"

}

# usage: add_senders_to_receiver RECEIVER_TEMPLATE
function add_senders_to_receiver
{
    CONFIG="$1"
    TEMPLATE="${TEMPLATES_DIR}/receiver-accepthost.conf"
    ANOTHER=""
    echo "Receiver accepts messages only from preconfigured senders:"
    while true ; do
        yes_or_no "Add$ANOTHER sender" ANSWER
        [ "$ANSWER" = no ] && return
        ANOTHER=" another"
        append_file "$TEMPLATE" "$CONFIG"
        set_existing_hostname "Sender hostname" XAHOSTNAME "$CONFIG"
        set_string "Sender certificate Common Name" XACOMMONNAME "$CONFIG"
        set_num_default "$DEFAULT_SENDER_PERIOD" \
              "Sender period for limit (in seconds)" XAPERIOD "$CONFIG"
        set_num_default "$DEFAULT_SENDER_LIMIT" \
              "Sender limit (in bytes)" XALIMIT "$CONFIG"
    done
}

# installs receiver
function install_receiver
{
    echolog "Configuring receiver"

    RECEIVER_TEMPLATE="${TEMPLATES_DIR}/receiver.conf"
    
    section setup_keys_receiver
    
    THIS_HOSTNAME=$(hostname -f 2>/dev/null)
    
    log "hostname -f returned $THIS_HOSTNAME"
    
    if [ ! -z "$THIS_HOSTNAME" ] ; then
        set_existing_hostname_default "$THIS_HOSTNAME" "This receiver hostname" \
                          XRECEIVERHOSTNAME "$RECEIVER_TEMPLATE"
    else
        set_existing_hostname "Where to send gathered data (receiver hostname)" \
                          XRECEIVERHOSTNAME "$RECEIVER_TEMPLATE"
    fi
    
    set_num_default "$DEFAULT_SERVER_PORT" \
                    "Receiver listening port" XRECEIVERPORT "$RECEIVER_TEMPLATE"

    add_senders_to_receiver "$RECEIVER_TEMPLATE"

    add_to_modules "Module receiver  Receiver  \"modules/receiver.conf\""

    add_to_dataflow "Add receiver -> XAGGREGATORSERVER"

    copy_file "$RECEIVER_TEMPLATE" "${MODULES_DIR}/receiver.conf"

}

# usage: install_filter NAME CONFIG DATAFLOW1 [ DATAFLOW2 ... ]
function install_filter
{
    NAME="$1"
    CONFIG="$2"
    shift 2
    TEMPLATE="${TEMPLATES_DIR}/$NAME.conf"
    FILTER_CONFIG="${ECHELOG_CONFDIR}/${CONFIG}"
    add_to_modules "Module $NAME   Filter    \"$CONFIG\"" 
    for FLOW in "$@" ; do
        add_to_dataflow "Add $NAME -> $FLOW"
    done
    copy_file "$TEMPLATE" "${FILTER_CONFIG}"
}

function finish_config
{
    # this allows configurations without aggregator and/or filter
    sed_file XFILTERAGENT "$FILTERAGENT" \
             ${TEMPLATES_DIR}/echelog_dataflow.conf >/dev/null
    sed_file XAGGREGATORAGENT "$AGGREGATORAGENT" \
             ${TEMPLATES_DIR}/echelog_dataflow.conf >/dev/null
    sed_file XFILTERSERVER "$FILTERSERVER" \
             ${TEMPLATES_DIR}/echelog_dataflow.conf >/dev/null
    sed_file XAGGREGATORSERVER "$AGGREGATORSERVER" \
             ${TEMPLATES_DIR}/echelog_dataflow.conf >/dev/null

    CONFIG="${ECHELOG_CONFDIR}/echelog.conf"
    zero_file "$CONFIG"
    for i in programs modules dataflow ; do
        append_file "${TEMPLATES_DIR}/echelog_${i}.conf" \
                    "${CONFIG}"
        zero_file "${TEMPLATES_DIR}/echelog_${i}.conf"
        echo >>"${ECHELOG_CONFDIR}/echelog.conf"
    done

    echo
    chown_files "all configuration files" \
          "${ECHELOG_CONFDIR}"/modules/*conf "$STATE_FILE"
}

function install_magent_ps
{
    TEMPLATE="${TEMPLATES_DIR}/ps.conf"
    CONFIG="modules/ps.conf"
    yes_or_no "Install Miniagent Ps" ANSWER
    [ "$ANSWER" = no ] && return
    set_interval_default "$DEFAULT_MAGENT_PS_INT" \
           "Interval between snapshots" "$TEMPLATE"
    add_to_modules "Module ps   Ps    \"$CONFIG\"" 
    add_to_dataflow "Add ps -> XAGGREGATORAGENT"
    copy_file "$TEMPLATE" "${ECHELOG_CONFDIR}/${CONFIG}"
}

function install_magent_netstat
{
    TEMPLATE="${TEMPLATES_DIR}/netstat.conf"
    CONFIG="modules/netstat.conf"
    yes_or_no "Install Miniagent Netstat" ANSWER
    [ "$ANSWER" = no ] && return
    set_interval_default "$DEFAULT_MAGENT_NETSTAT_INT" \
            "Interval between snapshots" "$TEMPLATE"
    add_to_modules "Module netstat   NetStat   \"$CONFIG\"" 
    add_to_dataflow "Add netstat -> XAGGREGATORAGENT"
    copy_file "$TEMPLATE" "${ECHELOG_CONFDIR}/${CONFIG}"
}

function install_magent_w
{
    TEMPLATE="${TEMPLATES_DIR}/w.conf"
    CONFIG="modules/w.conf"
    yes_or_no "Install Miniagent W" ANSWER
    [ "$ANSWER" = no ] && return
    set_interval_default "$DEFAULT_MAGENT_W_INT" \
           "Interval between snapshots" "$TEMPLATE"
    yes_or_no "Send uptime info" ANSWER
    if [ "$ANSWER" = yes ] ; then
        ANSWER=1
    else
        ANSWER=0
    fi
    sed_file XUPTIME "$ANSWER" "$TEMPLATE"
    add_to_modules "Module users-w   Users-w   \"$CONFIG\"" 
    add_to_dataflow "Add users-w -> XAGGREGATORAGENT"
    copy_file "$TEMPLATE" "${ECHELOG_CONFDIR}/${CONFIG}"
}

function install_magent_utmp
{
    TEMPLATE="${TEMPLATES_DIR}/utmp.conf"
    CONFIG="modules/utmp.conf"
    yes_or_no "Install Miniagent Utmp" ANSWER
    [ "$ANSWER" = no ] && return
    set_interval_default "$DEFAULT_MAGENT_UTMP_INT" \
           "Interval between snapshots (in seconds)" "$TEMPLATE"
    add_to_modules "Module users-utmp   Users-utmp   \"$CONFIG\"" 
    add_to_dataflow "Add users-utmp -> XAGGREGATORAGENT"
    copy_file "$TEMPLATE" "${ECHELOG_CONFDIR}/${CONFIG}"
}

function add_getlog
{
    I=$(($I+1))
    install_getlog "log$I" "$@"
}
    

function install_getlog
{
    IDENT="$1"
    shift
    # GETLOG_FILES="$@"

    GETLOG_TEMPLATE="${TEMPLATES_DIR}/getlog-template.conf"
    TEMPLATE="${TEMPLATES_DIR}/getlog-${IDENT}.conf"
    CONFIG="modules/getlog-$IDENT.conf"
    copy_file "$GETLOG_TEMPLATE" "$TEMPLATE"
    
    set_interval_default "$DEFAULT_LOGFILE_INT" \
            "How often to check logfiles" "$TEMPLATE"
    sed_file XINSTANCENAME "$IDENT" "$TEMPLATE"
    for LOGF in "$@" ; do
        append_to_file "LogFile $LOGF" "$TEMPLATE"
    done

    add_to_modules "Module getlog-$IDENT  GetLog   \"$CONFIG\"" 
    add_to_dataflow "Add getlog-$IDENT -> XFILTERAGENT"
    copy_file "$TEMPLATE" "${ECHELOG_CONFDIR}/${CONFIG}"
}

function install_predefined_getlogs
{
    for GL in $GETLOGS ; do
        eval POSSIBLE_LOGS="\"\$POSSIBLE_${GL}_LOGS\""
        FOUND_LOGS=""
        for L in $POSSIBLE_LOGS ; do
            if test -f "$L" ; then
                FOUND_LOGS="$FOUND_LOGS $L"
            fi
        done
        [ -z "$FOUND_LOGS" ] && continue
        yes_or_no "Do you want getlog-$GL to watch $FOUND_LOGS" ANSWER
        if [ "$ANSWER" = yes ] ; then
            install_getlog "$GL" $FOUND_LOGS
        fi
    done
}

function install_getlogs
{
    I=0
    yes_or_no "Do you want to watch any logfile (with Miniagent Getlog)" ANSWER
    [ "$ANSWER" = no ] && return
    echo
    echolog "WARNING: Set file permissions to allow getlog read logfiles."
    echo
    install_predefined_getlogs

# custom logs
    while true ; do
        yes_or_no "Do you want to add another logfiles" ANSWER
        [ "$ANSWER" = no ] && return
# TODO make this better/nicer
        ask_fname "Enter one or more logfiles (separated by spaces)" FLOG
        add_getlog $FLOG
    done
}

function install_binstor
{
    TEMPLATE="${TEMPLATES_DIR}/binstor.conf"
    CONFIG="modules/binstor.conf"
    
    log "Configuring BinStor module"
    echo -n "Configuring BinStor module ... "

    add_to_modules "Module binstor BinStor \"$CONFIG\""
    add_to_dataflow "Add binstor"
    copy_file "$TEMPLATE" "${ECHELOG_CONFDIR}/$CONFIG"

    echo "done"
}

# usage: add_alert_options OPTIONS
function add_alert_options
{
    ALERT_OPTIONS="${ALERT_OPTIONS} $@ "
    save_state ALERT_OPTIONS
}

# usage: set_alert_default_options CONFIG_FILE
function set_alert_default_options
{
    ALERT_OPTIONS="-C $1"
}

# usage: set_alert_logfile TEMPLATE
function set_alert_logfile
{
    TEMPLATE="$1"
    
    set_string_default "$DEFAULT_ALERT_LOGFILE" \
            "Log alerts to file" XLOGFILE "$TEMPLATE"
}

# usage: set_alert_email TEMPLATE
function set_alert_email
{
    TEMPLATE="$1"
    yes_or_no "Send email on alert" ANSWER
    [ "$ANSWER" = no ] && return
    
    set_string_default "$DEFAULT_ALERT_EMAIL" \
                       "Email address" XEMAIL_ADDRESS "$TEMPLATE"
    add_alert_options "-e"
}

# usage: set_alert_sms TEMPLATE
function set_alert_sms
{
    TEMPLATE="$1"
    yes_or_no "Send SMS on alert (you MUST have smssend installed)" ANSWER
    [ "$ANSWER" = no ] && return
    
    echo "Run tests if smssend supports your provider and if it really works."
    echo "(Run \"smssend -h\" and \"smssend <sms_provider> -help\" to find out how.)"
    echo "Remember also how smssend reports that SMS was succesfully accepted ..."

    yes_or_no "Does it really work" ANSWER
    [ "$ANSWER" = no ] && return
    
    set_string "smssend command line arguments" XSMSSEND_ARGS "$TEMPLATE"
    set_string_default "Okay.*sent" \
               "Smssend success response" XSMSSEND_OK "$TEMPLATE"
    set_string_default "10" "How many times to retry sending" \
                       XSMS_RETRY_NUM "$TEMPLATE"
    set_string_default "30" "Seconds between retries" \
                       XSMS_RETRY_INT "$TEMPLATE"
    yes_or_no "Send SMS through email when SMS failes" ANSWER
    if [ "$ANSWER" = "no" ] ; then
        sed_file XSMS_EMF no "$TEMPLATE"
    else
        sed_file XSMS_EMF yes "$TEMPLATE"
        set_string "SMS email address" XSMS_EMAIL "$TEMPLATE"
    fi
    add_alert_options "-s"
}

# usage: set_alert_write TEMPLATE
function set_alert_write
{
    TEMPLATE="$1"
    yes_or_no "Write alert to local user (standard unix \"write\")" ANSWER
    [ "$ANSWER" = no ] && return

    set_string_default "root" "Local user" XWRITE_USER "$TEMPLATE"
    add_alert_options "-w"
}

# usage: set_alert_xmessage TEMPLATE
function set_alert_xmessage
{
    TEMPLATE="$1"
    yes_or_no "Popup xmessage on remote display" ANSWER
    [ "$ANSWER" = no ] && return

    echo
    echolog "Warning: You must configure X display to accept connection from Echelog."
    echo

    set_string_default "$DEFAULT_ALERT_XDISPLAY" \
                       "Remote X display" XREMOTE_DISP "$TEMPLATE"
    add_alert_options "-x"
}

# usage: set_alert_othercmd TEMPLATE
function set_alert_othercmd
{
    TEMPLATE="$1"
    yes_or_no "Run other command on alert" ANSWER
    [ "$ANSWER" = no ] && return

    echo "The command will get the message on stdin."
    set_string "Command" XOTHER_CMD "$TEMPLATE"
    add_alert_options "-o"
}

# usage: set_alert_sound TEMPLATE
function set_alert_sound
{
    TEMPLATE="$1"
    yes_or_no "Play sound on alert" ANSWER
    [ "$ANSWER" = no ] && return

    echo
    echolog "Warning: You must configure the sound system to allow Echelog using it."
    echo

    set_existing_fname_default "$DEFAULT_ALERT_SOUND" \
             "Sound file (WAV)" XSOUND_FILE "$TEMPLATE"
    add_alert_options "-p"
}

# usage: install_alert TARGET ALERTMASTER_CONFIG
function install_alert
{
    ATARGET="$1"
    ALCONFIG="$2"
    ATEMP="${TEMPLATES_DIR}/alert.sh.local-conf"
    ATEMPNAME="alert.sh.${ATARGET}-conf"
    ATEMPLATE="${TEMPLATES_DIR}/${ATEMPNAME}"
    copy_file "$ATEMP" "$ATEMPLATE"
    GTEMPLATE="${TEMPLATES_DIR}/alert.sh.conf" # TODO find paths to programs?
    ACONFIG="${ECHELOG_CONFDIR}/modules/${ATEMPNAME}"
    GCONFIG="${ECHELOG_CONFDIR}/modules/alert.sh.conf"

    set_string_default "$DEFAULT_ALERT_BRIEFMSG" \
             "Alert subject (identification)" XBRIEFMSG "$ATEMPLATE"
    section set_alert_default_options "$ATEMPNAME"
    section set_alert_logfile "$ATEMPLATE"
    section set_alert_email "$ATEMPLATE"
    section set_alert_sms "$ATEMPLATE"
    section set_alert_write "$ATEMPLATE"
    section set_alert_xmessage "$ATEMPLATE"
    section set_alert_sound "$ATEMPLATE"
    section set_alert_othercmd "$ATEMPLATE"
    copy_file "$ATEMPLATE" "$ACONFIG"
    copy_file "$GTEMPLATE" "$GCONFIG"
    sed_file XALERT_OPTIONS "$ALERT_OPTIONS" "$ALCONFIG" 
}

# usage: install_alertmaster_agent FILTER FILTER-CONFIG
function install_alertmaster_agent
{
    install_alertmaster agent "$@"
}

# usage: install_alertmaster_server FILTER FILTER-CONFIG
function install_alertmaster_server
{
    install_alertmaster server "$@"
}

# usage: install_alertmaster TARGET FILTER FILTER-CONFIG
function install_alertmaster
{
    ALTARGET="$1"
    ALNAME="alertmaster-$ALTARGET"
    
    yes_or_no "Do you want to install alert example (alert on root login)" ANSWER
    [ "$ANSWER" = no ] && return

    AMTEMP="${TEMPLATES_DIR}/alertmaster.conf"
    CONFIG="modules/$ALNAME.conf"
    CONFILE="${ECHELOG_CONFDIR}/modules/$ALNAME.conf"
    copy_file "${AMTEMP}" "${CONFILE}"
    install_alert "$ALTARGET" "$CONFILE"
    
    F="$2"
    FC="$3"

    AMTEMPLATE="${TEMPLATES_DIR}/$ALNAME.conf"
    RULE="${TEMPLATES_DIR}/filter-rules-alert.conf"

    add_to_modules "Module $ALNAME AlertMaster \"$CONFIG\""
    add_to_dataflow "Add $F -> $ALNAME"
    add_to_dataflow "Add $ALNAME"
    insert_file "$RULE" "$FC" XALERTMASTER
    sed_file XALERTMASTERNAME "$ALNAME" "$FC"
}

function install_miniagents
{
    section install_magent_ps
    section install_magent_netstat
    section install_magent_w
    section install_magent_utmp
    section install_getlogs
}

# usage: install_filter_agent2 CONFIG DATAFLOW [ DATAFLOW... ]
function install_filter_agent2
{
    install_filter "$@"
}

# usage: install_filter_server2 CONFIG DATAFLOW [ DATAFLOW... ]
function install_filter_server2
{
    install_filter "$@"
}

function install_filter_agent
{
    yes_or_no "Install filter module (needed for local alert)" FILTER
    if [ "$FILTER" = yes ] ; then
        FILTERAGENT=filter-agent
        save_state FILTERAGENT
        FILTER_CONFIG="modules/$FILTERAGENT.conf"
        section install_filter_agent2 "$FILTERAGENT" "$FILTER_CONFIG" sender
        section install_alertmaster_agent "$FILTERAGENT" "$FILTER_CONFIG"
    else
        FILTERAGENT=sender
        save_state FILTERAGENT
    fi
}

# usage: install_logstor FILTERNAME FILTER_CONFIG_FILE
function install_logstor
{
    yes_or_no "Reconstruct plain text log files with LogStor" ANSWER
    [ "$ANSWER" = no ] && return

    FILTERNAME="$1"
    FILTERCFG="$2"

    LOGSTOR_RULE="${TEMPLATES_DIR}/filter-rules-logstor.conf"
    LOGSTORCFG="logstor.conf"

#    choose "Which type of messages do you want to log" MSGTYPE \
#           SessionMessage \
#           ProcessMessage \
#           NetstatMessage \
#           UptimeMessage \
#           SimpleLog
    
    MSGTYPE="SimpleLog"

    insert_file "$LOGSTOR_RULE" "$FILTERCFG" XLOGSTOR
    sed_file XLOGSTORMESSAGE "$MSGTYPE" "$FILTERCFG"
    copy_file "${TEMPLATES_DIR}/${LOGSTORCFG}" \
              "${ECHELOG_CONFDIR}/modules/${LOGSTORCFG}"

    add_to_dataflow "Add logstor"
    add_to_dataflow "Add $FILTERNAME -> logstor"
    add_to_modules  "Module logstor LogStor \"modules/logstor.conf\""
}

# usage: install_msgsniff FILTERNAME FILTER_CONFIG_FILE
function install_msgsniff
{
    yes_or_no "Log incoming messages with MsgSniff (extensive)" ANSWER
    [ "$ANSWER" = no ] && return

    FILTERNAME="$1"
    FILTERCFG="$2"

    MSGSNIFF_RULE="${TEMPLATES_DIR}/filter-rules-msgsniff.conf"
    MSGSNIFFCFG="msgsniff.conf"

    choose "Which type of messages do you want to sniff" MSGTYPE \
           SessionMessage \
           ProcessMessage \
           NetstatMessage \
           UptimeMessage \
           SimpleLog
    
    insert_file "$MSGSNIFF_RULE" "$FILTERCFG" XMSGSNIFF
    sed_file XMSGSNIFFMESSAGE "$MSGTYPE" "$FILTERCFG"
    copy_file "${TEMPLATES_DIR}/${MSGSNIFFCFG}" \
              "${ECHELOG_CONFDIR}/modules/${MSGSNIFFCFG}"

    add_to_dataflow "Add msgsniff"
    add_to_dataflow "Add $FILTERNAME -> msgsniff"
    add_to_modules  "Module msgsniff MsgSniff \"modules/msgsniff.conf\""
}

function install_filter_server
{
    yes_or_no "Install Filter module (required for local Alert, LogStor and MsgSniff)" FILTER
    if [ "$FILTER" = yes ] ; then
        FILTERSERVER=filter-server
        save_state FILTERSERVER
        FILTER_CONFIG="modules/$FILTERSERVER.conf"
        section install_filter_server2 "$FILTERSERVER" "$FILTER_CONFIG" binstor
        section install_alertmaster_server "$FILTERSERVER" "$FILTER_CONFIG"
        section install_logstor "$FILTERSERVER" "$FILTER_CONFIG"
        section install_msgsniff "$FILTERSERVER" "$FILTER_CONFIG"
    else
        FILTERSERVER=binstor
        save_state FILTERSERVER
    fi
}

# usage: install_aggregator TARGET CONFIG_FILE
function install_aggregator
{
    AGGTARGET="$1"
    CONFIG="$2"

    TEMPLATE="${TEMPLATES_DIR}/aggregator.conf"
    copy_file "$TEMPLATE" "$CONFIG"

    set_num_default "$DEFAULT_PS_AGG_SNAPSHOT" \
       "Miniagent Ps ProcessMessage snapshot interval (in seconds)" \
       XSN_PROCESS "$CONFIG"
    set_num_default "$DEFAULT_SESSION_AGG_SNAPSHOT" \
       "Miniagent W/Utmp SessionMessage snapshot interval (in seconds)" \
       XSN_SESSION "$CONFIG"
    set_num_default "$DEFAULT_NETSTAT_AGG_SNAPSHOT" \
       "Magent_netstat NetstatMessage snapshot interval (in seconds)" \
       XSN_NETSTAT "$CONFIG"
}

function install_aggregator_agent
{
    yes_or_no \
        "Install aggregator (recommended for Ps/W/Netstat miniagents)" ANSWER 
    if [ "$ANSWER" = yes ] ; then
        AGGREGATORAGENT=aggregator-agent
        save_state AGGREGATORAGENT
    else
        AGGREGATORAGENT="$FILTERAGENT"
        save_state AGGREGATORAGENT
        return
    fi
    install_aggregator "$AGGREGATORAGENT" \
                       "${ECHELOG_CONFDIR}/modules/${AGGREGATORAGENT}.conf"
    add_to_dataflow "Add $AGGREGATORAGENT -> XFILTERAGENT"
    add_to_modules "Module $AGGREGATORAGENT  Aggregator  \"modules/$AGGREGATORAGENT.conf\""
}

function install_aggregator_server
{
    yes_or_no \
        "Install aggregator (not needed, better on agents)" ANSWER 
    if [ "$ANSWER" = yes ] ; then
        AGGREGATORSERVER=aggregator-server
        save_state AGGREGATORSERVER
    else
        AGGREGATORSERVER="$FILTERSERVER"
        save_state AGGREGATORSERVER
        return
    fi
    install_aggregator "$AGGREGATORSERVER" \
                       "${ECHELOG_CONFDIR}/modules/${AGGREGATORSERVER}.conf"
    add_to_dataflow "Add $AGGREGATORSERVER -> XFILTERSERVER"
    add_to_modules "Module $AGGREGATORSERVER  Aggregator  \"modules/$AGGREGATORSERVER.conf\""
}

function install_webtools
{
    yes_or_no "Install WebTools for prezentation of logged data" ANSWER
    [ "$ANSWER" = no ] && return
    yes_or_no "Do you have Apache with PHP4 and RRDtool support" ANSWER
    [ "$ANSWER" = no ] && abort "Install it and then run this script again."

    echo
    echolog "                      WARNING!"
    echolog "WebTools are insecure because they allow running scripts"
    echolog "and reading local files through web interface."
    echolog "You should restrict the access to WebTools only to trusted IP's,"
    echolog "through password protected https!"
    echo

    
    WEBSOURCE="${TEMPLATES_DIR}/webtools"
    while true ; do
        [ -d "$WEBSOURCE" -a -f "${WEBSOURCE}/index.php" ] && break
        error "WebTools not found in $WEBSOURCE directory."
        ask_existing_dir "Location with sources of WebTools scripts" WEBSOURCE
    done

    umask 022

    while true ; do
        ask_string "Install WebTools scripts in (web accessible) system directory" WEBDIR
        WEBDIR="${WEBDIR}/echelog"
        mkdir -p "${WEBDIR}/tmp" && break
        error "Cannot create or write to $WEBDIR."
    done

    echo
    echolog "Make sure that Apache can write to \"${WEBDIR}/tmp\" directory."
    echolog "It is world-writable now, you should restrict it."
    echo
    chmod 777 "${WEBDIR}/tmp"

    TEMPLATE="${TEMPLATES_DIR}/config.inc"

    sed_file XLOCALDIR "${WEBDIR}" "$TEMPLATE"
    set_string_default "/echelog" "Directory part of the URL after \
\"http://local_server\" corresponding to $WEBDIR " XWEBDIR "$TEMPLATE"

    echolog "Copying WebTools from $WEBSOURCE to $WEBDIR"
    cp "${WEBSOURCE}"/*.php "${WEBSOURCE}"/*.inc "${WEBSOURCE}"/*.xsl \
       "$WEBDIR" || abort "Cannot copy"

    copy_file "${TEMPLATE}" "${WEBDIR}/config.inc"

    umask 077
}

function install_agent
{
    echo
    echolog " *** Configuring agent *** "
    echo
    add_to_modules  "# Agent part"
    add_to_dataflow "# Agent part"
    section install_sender
    section install_miniagents
    section install_filter_agent
    section install_aggregator_agent
}

function install_server
{
    echo
    echolog " *** Configuring server *** "
    echo
    add_to_modules  "# Server part"
    add_to_dataflow "# Server part"
    section install_receiver
    section install_binstor
    section install_filter_server
    section install_aggregator_server
    section install_webtools
}

function install_both
{
    section install_agent
    section install_server
}

function install_proxy
{
    section install_receiver
    section install_sender

# we want receiver -> sender dataflow
    sed_file XAGGREGATORSERVER "sender" \
             "${TEMPLATES_DIR}/echelog_dataflow.conf" >/dev/null
}

function choose_target
{
    choose "Configure Echelog on this computer as" GLOBALTARGET \
                                                 agent server both proxy
    save_state GLOBALTARGET
}

# usage: configure_echelog TARGET
function configure_echelog
{
    TARGET="$1"
    case "$TARGET" in
        agent)
            section install_agent
            ;;
        server)
            section install_server
            ;;
        both)
            section install_both
            ;;
        proxy)
            section install_proxy
            ;;
        *)
            abort "\"$TARGET\" installation target is not supported (bug?)"
            ;;
    esac

    section finish_config
}

function all_done
{
    echolog "Congratulations - Echelog is ready to run."
    log "FINISHED"
}

################
# INSTALLATION #
################

# initialization
init_conf
intro
set_echelog_confdir
set_logfile
set_all_paths
set_installation_state

restore_configuration
backup_configuration

section choose_target
section configure_echelog "$GLOBALTARGET"

all_done

cleanup
